@@ -19,11 +19,12 @@
 
 #include <haproxy/activity.h>
 #include <haproxy/api.h>
+#include <haproxy/clock.h>
 #include <haproxy/fd.h>
 #include <haproxy/global.h>
 #include <haproxy/signal.h>
+#include <haproxy/task.h>
 #include <haproxy/ticks.h>
-#include <haproxy/time.h>
 
 
 /* private data */
@@ -125,6 +126,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		changes = _update_fd(fd, changes);
 	}
 
+	thread_idle_now();
 	thread_harmless_now();
 
 	if (changes) {
@@ -141,11 +143,10 @@ static void _do_poll(struct poller *p, int exp, int wake)
 	}
 	fd_nbupdt = 0;
 
-	/* Now let's wait for polled events. */
+	/* now let's wait for events */
 	wait_time = wake ? 0 : compute_poll_timeout(exp);
 	fd = global.tune.maxpollevents;
-	tv_entering_poll();
-	activity_count_runtime();
+	clock_entering_poll();
 
 	do {
 		int timeout = (global.tune.options & GTUNE_BUSY_POLLING) ? 0 : wait_time;
@@ -159,7 +160,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		                kev,       // struct kevent *eventlist
 		                fd,        // int nevents
 		                &timeout_ts); // const struct timespec *timeout
-		tv_update_date(timeout, (global.tune.options & GTUNE_BUSY_POLLING) ? 1 : status);
+		clock_update_date(timeout, status);
 
 		if (status) {
 			activity[tid].poll_io++;
@@ -167,37 +168,29 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		}
 		if (timeout || !wait_time)
 			break;
-		if (wake)
+		if (signal_queue_len || wake)
 			break;
 		if (tick_isset(exp) && tick_is_expired(exp, now_ms))
 			break;
 	} while (1);
 
-	tv_leaving_poll(wait_time, status);
+	clock_leaving_poll(wait_time, status);
 
 	thread_harmless_end();
+	thread_idle_end();
+
 	if (sleeping_thread_mask & tid_bit)
 		_HA_ATOMIC_AND(&sleeping_thread_mask, ~tid_bit);
 
 	for (count = 0; count < status; count++) {
 		unsigned int n = 0;
+		int ret;
+
 		fd = kev[count].ident;
 
 #ifdef DEBUG_FD
 		_HA_ATOMIC_INC(&fdtab[fd].event_count);
 #endif
-		if (!fdtab[fd].owner) {
-			activity[tid].poll_dead_fd++;
-			continue;
-		}
-
-		if (!(fdtab[fd].thread_mask & tid_bit)) {
-			activity[tid].poll_skip_fd++;
-			if (!HA_ATOMIC_BTS(&fdtab[fd].update_mask, tid))
-				fd_updt[fd_nbupdt++] = fd;
-			continue;
-		}
-
 		if (kev[count].filter == EVFILT_READ) {
 			if (kev[count].data || !(kev[count].flags & EV_EOF))
 				n |= FD_EV_READY_R;
@@ -210,7 +203,13 @@ static void _do_poll(struct poller *p, int exp, int wake)
 				n |= FD_EV_ERR_RW;
 		}
 
-		fd_update_events(fd, n);
+		ret = fd_update_events(fd, n);
+
+		if (ret == FD_UPDT_MIGRATED) {
+			/* FD was migrated, let's stop polling it */
+			if (!HA_ATOMIC_BTS(&fdtab[fd].update_mask, tid))
+				fd_updt[fd_nbupdt++] = fd;
+		}
 	}
 }
 
