@@ -236,7 +236,7 @@ static void hc_cli_release(struct appctx *appctx)
 
 /* register cli keywords */
 static struct cli_kw_list cli_kws = {{ },{
-	{ { "httpclient", NULL }, "httpclient <method> <URI>   : launch an HTTP request", hc_cli_parse, hc_cli_io_handler, hc_cli_release,  NULL, ACCESS_EXPERT},
+	{ { "httpclient", NULL }, "httpclient <method> <URI>               : launch an HTTP request", hc_cli_parse, hc_cli_io_handler, hc_cli_release,  NULL, ACCESS_EXPERT},
 	{ { NULL }, NULL, NULL, NULL }
 }};
 
@@ -352,9 +352,10 @@ error:
  */
 int httpclient_res_xfer(struct httpclient *hc, struct buffer *dst)
 {
+	size_t room = b_room(dst);
 	int ret;
 
-	ret = b_force_xfer(dst, &hc->res.buf, MIN(1024, b_data(&hc->res.buf)));
+	ret = b_force_xfer(dst, &hc->res.buf, MIN(room, b_data(&hc->res.buf)));
 	/* call the client once we consumed all data */
 	if (!b_data(&hc->res.buf)) {
 		b_free(&hc->res.buf);
@@ -423,6 +424,13 @@ struct appctx *httpclient_start(struct httpclient *hc)
 	int len;
 	struct split_url out;
 
+	/* if the client was started and not ended, an applet is already
+	 * running, we shouldn't try anything */
+	if (httpclient_started(hc) && !httpclient_ended(hc))
+		return NULL;
+
+	hc->flags = 0;
+
 	/* parse URI and fill sockaddr_storage */
 	/* FIXME: use a resolver */
 	len = url2sa(istptr(hc->req.url), istlen(hc->req.url), &hc->dst, &out);
