@@ -27,7 +27,6 @@
 #include <haproxy/stream_interface.h>
 #include <haproxy/tcp_rules.h>
 #include <haproxy/ticks.h>
-#include <haproxy/time.h>
 #include <haproxy/tools.h>
 #include <haproxy/trace.h>
 
@@ -95,6 +94,7 @@ struct action_kw *tcp_res_cont_action(const char *kw)
  */
 int tcp_inspect_request(struct stream *s, struct channel *req, int an_bit)
 {
+	struct list *def_rules, *rules;
 	struct session *sess = s->sess;
 	struct act_rule *rule;
 	int partial;
@@ -102,6 +102,9 @@ int tcp_inspect_request(struct stream *s, struct channel *req, int an_bit)
 
 	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 
+	def_rules = ((s->be->defpx && (an_bit == AN_REQ_INSPECT_FE || s->be->defpx != sess->fe->defpx)) ? &s->be->defpx->tcp_req.inspect_rules : NULL);
+	rules = &s->be->tcp_req.inspect_rules;
+
 	/* We don't know whether we have enough data, so must proceed
 	 * this way :
 	 * - iterate through all rules in their declaration order
@@ -112,14 +115,10 @@ int tcp_inspect_request(struct stream *s, struct channel *req, int an_bit)
 	 * - if one rule returns KO, then return KO
 	 */
 
-	if ((req->flags & (CF_SHUTR|CF_READ_ERROR)) || channel_full(req, global.tune.maxrewrite) ||
+	if ((req->flags & (CF_EOI|CF_SHUTR|CF_READ_ERROR)) || channel_full(req, global.tune.maxrewrite) ||
 	    si_rx_blocked_room(chn_prod(req)) ||
-	    !s->be->tcp_req.inspect_delay || tick_is_expired(s->rules_exp, now_ms)) {
+	    !s->be->tcp_req.inspect_delay || tick_is_expired(s->rules_exp, now_ms))
 		partial = SMP_OPT_FINAL;
-		/* Action may yield while the inspect_delay is not expired and there is no read error */
-		if ((req->flags & CF_READ_ERROR) || !s->be->tcp_req.inspect_delay || tick_is_expired(s->rules_exp, now_ms))
-			act_opts |= ACT_OPT_FINAL;
-	}
 	else
 		partial = 0;
 
@@ -131,12 +130,13 @@ int tcp_inspect_request(struct stream *s, struct channel *req, int an_bit)
 	if (s->current_rule) {
 		rule = s->current_rule;
 		s->current_rule = NULL;
-		if (s->current_rule_list == &s->be->tcp_req.inspect_rules)
+		if ((def_rules && s->current_rule_list == def_rules) || s->current_rule_list == rules)
 			goto resume_execution;
 	}
-	s->current_rule_list = &s->be->tcp_req.inspect_rules;
+	s->current_rule_list = ((!def_rules || s->current_rule_list == def_rules) ? rules : def_rules);
 
-	list_for_each_entry(rule, &s->be->tcp_req.inspect_rules, list) {
+  restart:
+	list_for_each_entry(rule, s->current_rule_list, list) {
 		enum acl_test_res ret = ACL_TEST_PASS;
 
 		if (rule->cond) {
@@ -152,8 +152,12 @@ int tcp_inspect_request(struct stream *s, struct channel *req, int an_bit)
 		if (ret) {
 			act_opts |= ACT_OPT_FIRST;
 resume_execution:
+
 			/* Always call the action function if defined */
 			if (rule->action_ptr) {
+				if (partial & SMP_OPT_FINAL)
+					act_opts |= ACT_OPT_FINAL;
+
 				switch (rule->action_ptr(rule, s->be, s->sess, s, act_opts)) {
 					case ACT_RET_CONT:
 						break;
@@ -162,7 +166,7 @@ resume_execution:
 						goto end;
 					case ACT_RET_YIELD:
 						s->current_rule = rule;
-						if (act_opts & ACT_OPT_FINAL) {
+						if (partial & SMP_OPT_FINAL) {
 							send_log(s->be, LOG_WARNING,
 								 "Internal error: yield not allowed if the inspect-delay expired "
 								 "for the tcp-request content actions.");
@@ -191,11 +195,17 @@ resume_execution:
 		}
 	}
 
+	if (def_rules && s->current_rule_list == def_rules) {
+		s->current_rule_list = rules;
+		goto restart;
+	}
+
  end:
 	/* if we get there, it means we have no rule which matches, or
 	 * we have an explicit accept, so we apply the default accept.
 	 */
 	req->analysers &= ~an_bit;
+	s->current_rule = s->current_rule_list = NULL;
 	req->analyse_exp = s->rules_exp = TICK_ETERNITY;
 	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 	return 1;
@@ -235,6 +245,8 @@ resume_execution:
 
  abort:
 	req->analysers &= AN_REQ_FLT_END;
+	s->current_rule = s->current_rule_list = NULL;
+	req->analyse_exp = s->rules_exp = TICK_ETERNITY;
 
 	if (!(s->flags & SF_ERR_MASK))
 		s->flags |= SF_ERR_PRXCOND;
@@ -252,6 +264,7 @@ resume_execution:
  */
 int tcp_inspect_response(struct stream *s, struct channel *rep, int an_bit)
 {
+	struct list *def_rules, *rules;
 	struct session *sess = s->sess;
 	struct act_rule *rule;
 	int partial;
@@ -259,6 +272,9 @@ int tcp_inspect_response(struct stream *s, struct channel *rep, int an_bit)
 
 	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 
+	def_rules = (s->be->defpx ? &s->be->defpx->tcp_rep.inspect_rules : NULL);
+	rules = &s->be->tcp_rep.inspect_rules;
+
 	/* We don't know whether we have enough data, so must proceed
 	 * this way :
 	 * - iterate through all rules in their declaration order
@@ -268,14 +284,10 @@ int tcp_inspect_response(struct stream *s, struct channel *rep, int an_bit)
 	 * - if one rule returns OK, then return OK
 	 * - if one rule returns KO, then return KO
 	 */
-	if ((rep->flags & (CF_SHUTR|CF_READ_ERROR)) || channel_full(rep, global.tune.maxrewrite) ||
+	if ((rep->flags & (CF_EOI|CF_SHUTR|CF_READ_ERROR)) || channel_full(rep, global.tune.maxrewrite) ||
 	    si_rx_blocked_room(chn_prod(rep)) ||
-	    !s->be->tcp_rep.inspect_delay || tick_is_expired(s->rules_exp, now_ms)) {
+	    !s->be->tcp_rep.inspect_delay || tick_is_expired(s->rules_exp, now_ms))
 		partial = SMP_OPT_FINAL;
-		/* Action may yield while the inspect_delay is not expired and there is no read error */
-		if ((rep->flags & CF_READ_ERROR) || !s->be->tcp_rep.inspect_delay || tick_is_expired(s->rules_exp, now_ms))
-			act_opts |= ACT_OPT_FINAL;
-	}
 	else
 		partial = 0;
 
@@ -287,13 +299,13 @@ int tcp_inspect_response(struct stream *s, struct channel *rep, int an_bit)
 	if (s->current_rule) {
 		rule = s->current_rule;
 		s->current_rule = NULL;
-		if (s->current_rule_list == &s->be->tcp_rep.inspect_rules)
+		if ((def_rules && s->current_rule_list == def_rules) || s->current_rule_list == rules)
 			goto resume_execution;
 	}
-	s->current_rule_list = &s->be->tcp_rep.inspect_rules;
-	s->rules_exp = TICK_ETERNITY;
+	s->current_rule_list = ((!def_rules || s->current_rule_list == def_rules) ? rules : def_rules);
 
-	list_for_each_entry(rule, &s->be->tcp_rep.inspect_rules, list) {
+  restart:
+	list_for_each_entry(rule, s->current_rule_list, list) {
 		enum acl_test_res ret = ACL_TEST_PASS;
 
 		if (rule->cond) {
@@ -311,6 +323,9 @@ int tcp_inspect_response(struct stream *s, struct channel *rep, int an_bit)
 resume_execution:
 			/* Always call the action function if defined */
 			if (rule->action_ptr) {
+				if (partial & SMP_OPT_FINAL)
+					act_opts |= ACT_OPT_FINAL;
+
 				switch (rule->action_ptr(rule, s->be, s->sess, s, act_opts)) {
 					case ACT_RET_CONT:
 						break;
@@ -319,12 +334,13 @@ resume_execution:
 						goto end;
 					case ACT_RET_YIELD:
 						s->current_rule = rule;
-						if (act_opts & ACT_OPT_FINAL) {
+						if (partial & SMP_OPT_FINAL) {
 							send_log(s->be, LOG_WARNING,
 								 "Internal error: yield not allowed if the inspect-delay expired "
 								 "for the tcp-response content actions.");
 							goto internal;
 						}
+						channel_dont_close(rep);
 						goto missing_data;
 					case ACT_RET_DENY:
 						goto deny;
@@ -355,18 +371,23 @@ resume_execution:
 		}
 	}
 
+	if (def_rules && s->current_rule_list == def_rules) {
+		s->current_rule_list = rules;
+		goto restart;
+	}
+
  end:
 	/* if we get there, it means we have no rule which matches, or
 	 * we have an explicit accept, so we apply the default accept.
 	 */
 	rep->analysers &= ~an_bit;
+	s->current_rule = s->current_rule_list = NULL;
 	rep->analyse_exp = s->rules_exp = TICK_ETERNITY;
 	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 	return 1;
 
  missing_data:
 	/* just set the analyser timeout once at the beginning of the response */
-	channel_dont_close(rep);
 	if (!tick_isset(s->rules_exp) && s->be->tcp_rep.inspect_delay)
 		s->rules_exp = tick_add(now_ms, s->be->tcp_rep.inspect_delay);
 	rep->analyse_exp = tick_first((tick_is_expired(rep->analyse_exp, now_ms) ? 0 : rep->analyse_exp), s->rules_exp);
@@ -405,6 +426,8 @@ resume_execution:
 
   abort:
 	rep->analysers &= AN_RES_FLT_END;
+	s->current_rule = s->current_rule_list = NULL;
+	rep->analyse_exp = s->rules_exp = TICK_ETERNITY;
 
 	if (!(s->flags & SF_ERR_MASK))
 		s->flags |= SF_ERR_PRXCOND;
@@ -422,6 +445,7 @@ resume_execution:
  */
 int tcp_exec_l4_rules(struct session *sess)
 {
+	struct proxy *px = sess->fe;
 	struct act_rule *rule;
 	struct connection *conn = objt_conn(sess->origin);
 	int result = 1;
@@ -430,7 +454,11 @@ int tcp_exec_l4_rules(struct session *sess)
 	if (!conn)
 		return result;
 
-	list_for_each_entry(rule, &sess->fe->tcp_req.l4_rules, list) {
+	if  (sess->fe->defpx)
+		px = sess->fe->defpx;
+
+  restart:
+	list_for_each_entry(rule, &px->tcp_req.l4_rules, list) {
 		ret = ACL_TEST_PASS;
 
 		if (rule->cond) {
@@ -498,6 +526,11 @@ int tcp_exec_l4_rules(struct session *sess)
 			}
 		}
 	}
+
+	if (px != sess->fe) {
+		px = sess->fe;
+		goto restart;
+	}
  end:
 	return result;
 }
@@ -511,11 +544,16 @@ int tcp_exec_l4_rules(struct session *sess)
  */
 int tcp_exec_l5_rules(struct session *sess)
 {
+	struct proxy *px = sess->fe;
 	struct act_rule *rule;
 	int result = 1;
 	enum acl_test_res ret;
 
-	list_for_each_entry(rule, &sess->fe->tcp_req.l5_rules, list) {
+	if  (sess->fe->defpx)
+		px = sess->fe->defpx;
+
+  restart:
+	list_for_each_entry(rule, &px->tcp_req.l5_rules, list) {
 		ret = ACL_TEST_PASS;
 
 		if (rule->cond) {
@@ -565,6 +603,11 @@ int tcp_exec_l5_rules(struct session *sess)
 			}
 		}
 	}
+
+	if (px != sess->fe) {
+		px = sess->fe;
+		goto restart;
+	}
   end:
 	return result;
 }
@@ -576,8 +619,8 @@ static int tcp_parse_response_rule(char **args, int arg, int section_type,
                                    unsigned int where,
                                    const char *file, int line)
 {
-	if (curpx == defpx || !(curpx->cap & PR_CAP_BE)) {
-		memprintf(err, "%s %s is only allowed in 'backend' sections",
+	if ((curpx == defpx && strlen(defpx->id) == 0) || !(curpx->cap & PR_CAP_BE)) {
+		memprintf(err, "%s %s is only allowed in 'backend' sections or 'defaults' section with a name",
 		          args[0], args[1]);
 		return -1;
 	}
@@ -744,9 +787,9 @@ static int tcp_parse_request_rule(char **args, int arg, int section_type,
                                   struct act_rule *rule, char **err,
                                   unsigned int where, const char *file, int line)
 {
-	if (curpx == defpx) {
-		memprintf(err, "%s %s is not allowed in 'defaults' sections",
-		          args[0], args[1]);
+	if (curpx == defpx && strlen(defpx->id) == 0) {
+		memprintf(err, "%s %s is not allowed in anonymous 'defaults' sections",
+			  args[0], args[1]);
 		return -1;
 	}
 
@@ -1033,8 +1076,8 @@ static int tcp_parse_tcp_rep(char **args, int section_type, struct proxy *curpx,
 	}
 
 	if (strcmp(args[1], "inspect-delay") == 0) {
-		if (curpx == defpx || !(curpx->cap & PR_CAP_BE)) {
-			memprintf(err, "%s %s is only allowed in 'backend' sections",
+		if ((curpx == defpx && strlen(defpx->id) == 0) || !(curpx->cap & PR_CAP_BE)) {
+			memprintf(err, "%s %s is only allowed in 'backend' sections or 'defaults' section with a name",
 			          args[0], args[1]);
 			return -1;
 		}
@@ -1062,7 +1105,7 @@ static int tcp_parse_tcp_rep(char **args, int section_type, struct proxy *curpx,
 		return 0;
 	}
 
-	rule = calloc(1, sizeof(*rule));
+	rule = new_act_rule(ACT_F_TCP_RES_CNT, file, line);
 	if (!rule) {
 		memprintf(err, "parsing [%s:%d] : out of memory", file, line);
 		return -1;
@@ -1078,7 +1121,6 @@ static int tcp_parse_tcp_rep(char **args, int section_type, struct proxy *curpx,
 			where |= SMP_VAL_FE_RES_CNT;
 		if (curpx->cap & PR_CAP_BE)
 			where |= SMP_VAL_BE_RES_CNT;
-		rule->from = ACT_F_TCP_RES_CNT;
 		if (tcp_parse_response_rule(args, arg, section_type, curpx, defpx, rule, err, where, file, line) < 0)
 			goto error;
 
@@ -1151,9 +1193,9 @@ static int tcp_parse_tcp_req(char **args, int section_type, struct proxy *curpx,
 	}
 
 	if (strcmp(args[1], "inspect-delay") == 0) {
-		if (curpx == defpx) {
-			memprintf(err, "%s %s is not allowed in 'defaults' sections",
-			          args[0], args[1]);
+		if (curpx == defpx && strlen(defpx->id) == 0) {
+			memprintf(err, "%s %s is not allowed in anonymous 'defaults' sections",
+				  args[0], args[1]);
 			return -1;
 		}
 
@@ -1180,7 +1222,7 @@ static int tcp_parse_tcp_req(char **args, int section_type, struct proxy *curpx,
 		return 0;
 	}
 
-	rule = calloc(1, sizeof(*rule));
+	rule = new_act_rule(0, file, line);
 	if (!rule) {
 		memprintf(err, "parsing [%s:%d] : out of memory", file, line);
 		return -1;
